ARPACK = lapack/dnaupd.c lapack/dnaup2.c lapack/dgetv0.c lapack/dvout.c lapack/second.c lapack/dmout.c lapack/dnaitr.c lapack/ivout.c lapack/dnapps.c lapack/dnconv.c lapack/dneigh.c lapack/dlaqrb.c lapack/dngets.c lapack/dsortc.c lapack/dstatn.c lapack/dneupd.c lapack/dsaupd.c lapack/dsaup2.c lapack/dsaitr.c lapack/dsapps.c lapack/dsconv.c lapack/dseigt.c lapack/dstqrb.c lapack/dsgets.c lapack/dsortr.c lapack/dstats.c lapack/dseupd.c lapack/dsesrt.c 
