LAPACK = lapack/dgeev.c lapack/dgebak.c lapack/xerbla.c lapack/dgebal.c lapack/disnan.c lapack/dlaisnan.c  lapack/dgehrd.c lapack/dgehd2.c lapack/dlarf.c lapack/iladlc.c lapack/iladlr.c lapack/dlarfg.c lapack/dlapy2.c lapack/dlahr2.c lapack/dlacpy.c lapack/dlarfb.c lapack/ilaenv.c lapack/ieeeck.c lapack/iparmq.c lapack/dhseqr.c lapack/dlahqr.c lapack/dlabad.c lapack/dlanv2.c lapack/dlaqr0.c lapack/dlaqr3.c lapack/dlaqr4.c lapack/dlaqr2.c lapack/dlaset.c lapack/dormhr.c lapack/dormqr.c lapack/dlarft.c lapack/dorm2r.c lapack/dtrexc.c lapack/dlaexc.c lapack/dlange.c lapack/dlassq.c lapack/dlarfx.c lapack/dlartg.c lapack/dlasy2.c lapack/dlaqr5.c lapack/dlaqr1.c lapack/dlascl.c lapack/dorghr.c lapack/dorgqr.c lapack/dorg2r.c lapack/dtrevc.c lapack/dlaln2.c lapack/dladiv.c lapack/dsyevr.c lapack/dlansy.c lapack/dormtr.c lapack/dormql.c lapack/dorm2l.c lapack/dstebz.c lapack/dlaebz.c lapack/dstein.c lapack/dlagtf.c lapack/dlagts.c lapack/dlarnv.c lapack/dlaruv.c lapack/dstemr.c lapack/dlae2.c lapack/dlaev2.c lapack/dlanst.c lapack/dlarrc.c lapack/dlarre.c lapack/dlarra.c lapack/dlarrb.c lapack/dlaneg.c lapack/dlarrd.c lapack/dlarrk.c lapack/dlasq2.c lapack/dlasq3.c lapack/dlasq4.c lapack/dlasq5.c lapack/dlasq6.c lapack/dlasrt.c lapack/dlarrj.c lapack/dlarrr.c lapack/dlarrv.c lapack/dlar1v.c lapack/dlarrf.c lapack/dsterf.c lapack/dsytrd.c lapack/dlatrd.c lapack/dsytd2.c lapack/dlanhs.c lapack/dgeqr2.c lapack/dtrsen.c lapack/dlacn2.c lapack/dtrsyl.c lapack/dlasr.c lapack/dsteqr.c lapack/dgeevx.c lapack/dtrsna.c lapack/dlaqtr.c lapack/dgetrf.c lapack/dgetf2.c lapack/dlaswp.c lapack/dgetrs.c lapack/dgesv.c lapack/dpotrf.c lapack/dpotf2.c lapack/len_trim.c 

