PRPACK = prpack/prpack_base_graph.cpp \
         prpack/prpack_igraph_graph.cpp \
         prpack/prpack_preprocessed_ge_graph.cpp \
         prpack/prpack_preprocessed_gs_graph.cpp \
         prpack/prpack_preprocessed_scc_graph.cpp \
         prpack/prpack_preprocessed_schur_graph.cpp \
         prpack/prpack_result.cpp \
         prpack/prpack_solver.cpp \
         prpack/prpack_utils.cpp \
         prpack/prpack.h \
         prpack/prpack_base_graph.h \
         prpack/prpack_csc.h \
         prpack/prpack_csr.h \
         prpack/prpack_edge_list.h \
         prpack/prpack_igraph_graph.h \
         prpack/prpack_preprocessed_ge_graph.h \
         prpack/prpack_preprocessed_graph.h \
         prpack/prpack_preprocessed_gs_graph.h \
         prpack/prpack_preprocessed_scc_graph.h \
         prpack/prpack_preprocessed_schur_graph.h \
         prpack/prpack_result.h \
         prpack/prpack_solver.h \
         prpack/prpack_utils.h
