PLFIT = plfit/error.c     plfit/gss.c       plfit/kolmogorov.c \
        plfit/lbfgs.c     plfit/options.c   plfit/plfit.c      \
		plfit/hzeta.c     plfit/mt.c        plfit/sampling.c   \
		plfit/rbinom.c    plfit/platform.c  \
		plfit/arithmetic_ansi.h plfit/arithmetic_sse_double.h plfit/arithmetic_sse_float.h \
		plfit/error.h     plfit/gss.h       plfit/kolmogorov.h \
		plfit/lbfgs.h     plfit/platform.h  plfit/plfit.h      \
		plfit/hzeta.h     plfit/mt.h        plfit/sampling.h


