/* -*- mode: C -*-  */
/*
   IGraph library.
   Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
   334 Harvard st, Cambridge MA, 02139 USA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301 USA

*/
#include <igraph.h>
#include <stdio.h>

int main(int argc, char **argv) {

    /* igraph_t g; */
    /* igraph_error_handler_t *oldhandler; */
    /* FILE *ofile; */
    /* int ret; */

    /* This is not used right now, as we don't have attributes */

    /* Testing error handling */
    /*   igraph_barabasi_game(&g, 10, 1, 0, 0, IGRAPH_DIRECTED); */
    /*   oldhandler=igraph_set_error_handler(igraph_error_handler_ignore); */
    /*   ofile=fopen("test.txt", "w"); */
    /*   ret=igraph_write_graph_lgl(&g, ofile, "names", "weights", 1); */
    /*   if (ret != IGRAPH_EINVAL) { */
    /*     return 1; */
    /*   } */
    /*   fclose(ofile); */
    /*   igraph_destroy(&g); */
    /*   igraph_set_error_handler(oldhandler); */

    return 0;
}
